from datetime import datetime
from typing import ClassVar, Dict, Optional, Type, List

from attr import define, field as attrs_field

from resoto_plugin_aws.aws_client import AwsClient
from resoto_plugin_aws.resource.base import AwsApiSpec, GraphBuilder, AwsResource
from resoto_plugin_aws.resource.cloudwatch import AwsCloudwatchLogGroup
from resoto_plugin_aws.resource.iam import AwsIamRole
from resoto_plugin_aws.resource.kms import AwsKmsKey
from resoto_plugin_aws.resource.s3 import AwsS3Bucket
from resoto_plugin_aws.resource.sns import AwsSnsTopic
from resoto_plugin_aws.utils import ToDict
from resotolib.graph import Graph
from resotolib.types import Json
from resotolib.baseresources import ModelReference, EdgeType
from resotolib.json import from_json
from resotolib.json_bender import Bender, S, bend, ForallBend, EmptyToNone, F

service_name = "cloudtrail"


@define(eq=False, slots=False)
class AwsCloudTrailAdvancedFieldSelector:
    kind: ClassVar[str] = "aws_cloud_trail_advanced_field_selector"
    mapping: ClassVar[Dict[str, Bender]] = {
        "field": S("Field"),
        "equals": S("Equals"),
        "starts_with": S("StartsWith"),
        "ends_with": S("EndsWith"),
        "not_equals": S("NotEquals"),
        "not_starts_with": S("NotStartsWith"),
        "not_ends_with": S("NotEndsWith"),
    }
    equals: Optional[List[str]] = attrs_field(default=None)
    starts_with: Optional[List[str]] = attrs_field(default=None)
    ends_with: Optional[List[str]] = attrs_field(default=None)
    not_equals: Optional[List[str]] = attrs_field(default=None)
    not_starts_with: Optional[List[str]] = attrs_field(default=None)
    not_ends_with: Optional[List[str]] = attrs_field(default=None)


@define(eq=False, slots=False)
class AwsCloudTrailEventSelector:
    kind: ClassVar[str] = "aws_cloud_trail_event_selector"
    mapping: ClassVar[Dict[str, Bender]] = {
        "name": S("Name"),
        "field_selectors": S("FieldSelectors", default=[])
        >> ForallBend(AwsCloudTrailAdvancedFieldSelector.mapping)
        >> F(lambda x: {a["field"]: a for a in x}),
    }
    name: Optional[str] = attrs_field(default=None)
    field_selectors: Optional[Dict[str, AwsCloudTrailAdvancedFieldSelector]] = attrs_field(default=None)


@define(eq=False, slots=False)
class AwsCloudTrailStatus:
    kind: ClassVar[str] = "aws_cloud_trail_status"
    mapping: ClassVar[Dict[str, Bender]] = {
        "is_logging": S("IsLogging"),
        "latest_delivery_error": S("LatestDeliveryError"),
        "latest_notification_error": S("LatestNotificationError"),
        "latest_delivery_time": S("LatestDeliveryTime") >> EmptyToNone,
        "latest_notification_time": S("LatestNotificationTime") >> EmptyToNone,
        "start_logging_time": S("StartLoggingTime") >> EmptyToNone,
        "stop_logging_time": S("StopLoggingTime") >> EmptyToNone,
        "latest_cloud_watch_logs_delivery_error": S("LatestCloudWatchLogsDeliveryError"),
        "latest_cloud_watch_logs_delivery_time": S("LatestCloudWatchLogsDeliveryTime"),
        "latest_digest_delivery_time": S("LatestDigestDeliveryTime") >> EmptyToNone,
        "latest_digest_delivery_error": S("LatestDigestDeliveryError"),
        "latest_delivery_attempt_time": S("LatestDeliveryAttemptTime") >> EmptyToNone,
        "latest_notification_attempt_time": S("LatestNotificationAttemptTime") >> EmptyToNone,
        "latest_notification_attempt_succeeded": S("LatestNotificationAttemptSucceeded") >> EmptyToNone,
        "latest_delivery_attempt_succeeded": S("LatestDeliveryAttemptSucceeded") >> EmptyToNone,
        "time_logging_started": S("TimeLoggingStarted") >> EmptyToNone,
        "time_logging_stopped": S("TimeLoggingStopped") >> EmptyToNone,
    }
    is_logging: Optional[bool] = attrs_field(default=None)
    latest_delivery_error: Optional[str] = attrs_field(default=None)
    latest_notification_error: Optional[str] = attrs_field(default=None)
    latest_delivery_time: Optional[datetime] = attrs_field(default=None)
    latest_notification_time: Optional[datetime] = attrs_field(default=None)
    start_logging_time: Optional[datetime] = attrs_field(default=None)
    stop_logging_time: Optional[datetime] = attrs_field(default=None)
    latest_cloud_watch_logs_delivery_error: Optional[str] = attrs_field(default=None)
    latest_cloud_watch_logs_delivery_time: Optional[datetime] = attrs_field(default=None)
    latest_digest_delivery_time: Optional[datetime] = attrs_field(default=None)
    latest_digest_delivery_error: Optional[str] = attrs_field(default=None)
    latest_delivery_attempt_time: Optional[datetime] = attrs_field(default=None)
    latest_notification_attempt_time: Optional[datetime] = attrs_field(default=None)
    latest_notification_attempt_succeeded: Optional[datetime] = attrs_field(default=None)
    latest_delivery_attempt_succeeded: Optional[datetime] = attrs_field(default=None)
    time_logging_started: Optional[datetime] = attrs_field(default=None)
    time_logging_stopped: Optional[datetime] = attrs_field(default=None)


@define(eq=False, slots=False)
class AwsCloudTrail(AwsResource):
    kind: ClassVar[str] = "aws_cloud_trail"
    api_spec: ClassVar[AwsApiSpec] = AwsApiSpec(service_name, "list-trails", "Trails")
    mapping: ClassVar[Dict[str, Bender]] = {
        "id": S("Name"),
        "name": S("Name"),
        "trail_s3_bucket_name": S("S3BucketName"),
        "trail_s3_key_prefix": S("S3KeyPrefix"),
        "trail_sns_topic_name": S("SnsTopicName"),
        "trail_sns_topic_arn": S("SnsTopicARN"),
        "trail_include_global_service_events": S("IncludeGlobalServiceEvents"),
        "trail_is_multi_region_trail": S("IsMultiRegionTrail"),
        "trail_home_region": S("HomeRegion"),
        "arn": S("TrailARN"),
        "trail_log_file_validation_enabled": S("LogFileValidationEnabled"),
        "trail_cloud_watch_logs_log_group_arn": S("CloudWatchLogsLogGroupArn"),
        "trail_cloud_watch_logs_role_arn": S("CloudWatchLogsRoleArn"),
        "trail_kms_key_id": S("KmsKeyId"),
        "trail_has_custom_event_selectors": S("HasCustomEventSelectors"),
        "trail_has_insight_selectors": S("HasInsightSelectors"),
        "trail_is_organization_trail": S("IsOrganizationTrail"),
    }
    reference_kinds: ClassVar[ModelReference] = {
        "successors": {"default": ["aws_s3_bucket", "aws_sns_topic", "aws_kms_key"]},
    }
    trail_s3_bucket_name: Optional[str] = attrs_field(default=None)
    trail_s3_key_prefix: Optional[str] = attrs_field(default=None)
    trail_sns_topic_name: Optional[str] = attrs_field(default=None)
    trail_sns_topic_arn: Optional[str] = attrs_field(default=None)
    trail_include_global_service_events: Optional[bool] = attrs_field(default=None)
    trail_is_multi_region_trail: Optional[bool] = attrs_field(default=None)
    trail_home_region: Optional[str] = attrs_field(default=None)
    arn: Optional[str] = attrs_field(default=None)
    trail_log_file_validation_enabled: Optional[bool] = attrs_field(default=None)
    trail_cloud_watch_logs_log_group_arn: Optional[str] = attrs_field(default=None)
    trail_cloud_watch_logs_role_arn: Optional[str] = attrs_field(default=None)
    trail_kms_key_id: Optional[str] = attrs_field(default=None)
    trail_has_custom_event_selectors: Optional[bool] = attrs_field(default=None)
    trail_has_insight_selectors: Optional[bool] = attrs_field(default=None)
    trail_is_organization_trail: Optional[bool] = attrs_field(default=None)
    trail_status: Optional[AwsCloudTrailStatus] = attrs_field(default=None)
    trail_event_selectors: Optional[List[AwsCloudTrailEventSelector]] = attrs_field(default=None)
    trail_insight_selectors: Optional[List[str]] = attrs_field(default=None)

    @classmethod
    def called_collect_apis(cls) -> List[AwsApiSpec]:
        return [
            AwsApiSpec(service_name, "list-trails"),
            AwsApiSpec(service_name, "get-trail"),
            AwsApiSpec(service_name, "get-trail-status"),
            AwsApiSpec(service_name, "list-tags"),
            AwsApiSpec(service_name, "get-event-selectors"),
            AwsApiSpec(service_name, "get-insight-selectors"),
        ]

    @classmethod
    def collect(cls: Type[AwsResource], json: List[Json], builder: GraphBuilder) -> None:
        def collect_trail(trail_arn: str) -> None:
            if trail_raw := builder.client.get(service_name, "get-trail", "Trail", Name=trail_arn):
                instance = AwsCloudTrail.from_api(trail_raw)
                builder.add_node(instance, js)
                collect_status(instance)
                collect_tags(instance)
                if instance.trail_has_custom_event_selectors:
                    collect_event_selectors(instance)
                if instance.trail_has_insight_selectors:
                    collect_insight_selectors(instance)

        def collect_event_selectors(trail: AwsCloudTrail) -> None:
            trail.trail_event_selectors = []
            for item in builder.client.list(
                service_name, "get-event-selectors", "AdvancedEventSelectors", TrailName=trail.arn
            ):
                mapped = bend(AwsCloudTrailEventSelector.mapping, item)
                trail.trail_event_selectors.append(from_json(mapped, AwsCloudTrailEventSelector))

        def collect_insight_selectors(trail: AwsCloudTrail) -> None:
            trail.trail_insight_selectors = []
            for item in builder.client.list(
                service_name,
                "get-insight-selectors",
                "InsightSelectors",
                TrailName=trail.arn,
                expected_errors=["InsightNotEnabledException"],
            ):
                trail.trail_insight_selectors.append(item["InsightType"])

        def collect_status(trail: AwsCloudTrail) -> None:
            status_raw = builder.client.get(service_name, "get-trail-status", Name=trail.arn)
            mapped = bend(AwsCloudTrailStatus.mapping, status_raw)
            status = from_json(mapped, AwsCloudTrailStatus)
            trail.trail_status = status
            trail.ctime = status.start_logging_time
            trail.mtime = status.latest_delivery_time

        def collect_tags(trail: AwsCloudTrail) -> None:
            for tr in builder.client.list(
                service_name,
                "list-tags",
                "ResourceTagList",
                ResourceIdList=[trail.arn],
                expected_errors=["CloudTrailARNInvalidException", "AccessDeniedException"],
            ):
                trail.tags = bend(S("TagsList", default=[]) >> ToDict(), tr)

        for js in json:
            arn = js["TrailARN"]
            # list trails will return multi account trails in all regions
            if js["HomeRegion"] == builder.region.name and builder.account.id in arn:
                # only collect trails in the current account and current region
                builder.submit_work(service_name, collect_trail, arn)
            else:
                # add a deferred edge to the trails in another account or region
                builder.add_deferred_edge(
                    builder.region, EdgeType.default, f'is(aws_cloud_trail) and reported.arn=="{arn}"'
                )

    def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
        if s3 := self.trail_s3_bucket_name:
            builder.add_edge(self, clazz=AwsS3Bucket, name=s3)
        if sns := self.trail_sns_topic_arn:
            builder.add_edge(self, clazz=AwsSnsTopic, arn=sns)
        if kms := self.trail_kms_key_id:
            builder.add_edge(self, clazz=AwsKmsKey, id=AwsKmsKey.normalise_id(kms))
        if log_group := self.trail_cloud_watch_logs_log_group_arn:
            builder.add_edge(self, clazz=AwsCloudwatchLogGroup, arn=log_group)
        if log_role := self.trail_cloud_watch_logs_role_arn:
            builder.add_edge(self, clazz=AwsIamRole, arn=log_role)

    def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
        client.call(service_name, "add-tags", ResourceId=self.arn, TagsList=[{"Key": key, "Value": value}])
        return True

    def delete_resource_tag(self, client: AwsClient, key: str) -> bool:
        client.call(service_name, "remove-tags", ResourceId=self.arn, TagsList=[{"Key": key}])
        return True

    def delete_resource(self, client: AwsClient, graph: Graph) -> bool:
        client.call(service_name, "delete-trail", Name=self.arn)
        return True

    @classmethod
    def called_mutator_apis(cls) -> List[AwsApiSpec]:
        return [
            AwsApiSpec(service_name, "add-tags"),
            AwsApiSpec(service_name, "remove-tags"),
            AwsApiSpec(service_name, "delete-trail"),
        ]


resources: List[Type[AwsResource]] = [AwsCloudTrail]



from resotolib.graph import Graph
from test.resources import round_trip_for
from types import SimpleNamespace
from typing import cast, Any
from resoto_plugin_aws.aws_client import AwsClient
from resoto_plugin_aws.resource.dynamodb import AwsDynamoDbTable, AwsDynamoDbGlobalTable


def test_tables() -> None:
    first, builder = round_trip_for(AwsDynamoDbTable)
    assert len(builder.resources_of(AwsDynamoDbTable)) == 1
    assert len(first.tags) == 1


def test_tagging_tables() -> None:
    table, _ = round_trip_for(AwsDynamoDbTable)

    def validate_update_args(**kwargs: Any) -> Any:
        if kwargs["action"] == "list-tags-of-resource":
            assert kwargs["ResourceArn"] == table.arn
            return [{"Key": "foo", "Value": "bar"}]

        if kwargs["action"] == "tag-resource":
            assert kwargs["ResourceArn"] == table.arn
            assert kwargs["Tags"] == [{"Key": "foo", "Value": "bar"}]

    def validate_delete_args(**kwargs: Any) -> Any:
        if kwargs["action"] == "untag-resource":
            assert kwargs["ResourceArn"] == table.arn
            assert kwargs["TagKeys"] == ["foo"]

    client = cast(AwsClient, SimpleNamespace(call=validate_update_args))
    table.update_resource_tag(client, "foo", "bar")

    client = cast(AwsClient, SimpleNamespace(call=validate_delete_args))
    table.delete_resource_tag(client, "foo")


def test_delete_tables() -> None:
    table, _ = round_trip_for(AwsDynamoDbTable)

    def validate_delete_args(**kwargs: Any) -> Any:
        assert kwargs["action"] == "delete-table"
        assert kwargs["TableName"] == table.name

    client = cast(AwsClient, SimpleNamespace(call=validate_delete_args))
    table.delete_resource(client, Graph())


def test_global_tables() -> None:
    first, builder = round_trip_for(AwsDynamoDbGlobalTable)
    assert len(builder.resources_of(AwsDynamoDbGlobalTable)) == 1


def test_tagging_global_tables() -> None:
    table, _ = round_trip_for(AwsDynamoDbGlobalTable)

    def validate_update_args(**kwargs: Any) -> Any:
        if kwargs["action"] == "list-tags-of-resource":
            assert kwargs["ResourceArn"] == table.arn
            return [{"Key": "foo", "Value": "bar"}]

        if kwargs["action"] == "tag-resource":
            assert kwargs["ResourceArn"] == table.arn
            assert kwargs["Tags"] == [{"Key": "foo", "Value": "bar"}]

    def validate_delete_args(**kwargs: Any) -> Any:
        if kwargs["action"] == "untag-resource":
            assert kwargs["ResourceArn"] == table.arn
            assert kwargs["TagKeys"] == ["foo"]

    client = cast(AwsClient, SimpleNamespace(call=validate_update_args))
    table.update_resource_tag(client, "foo", "bar")

    client = cast(AwsClient, SimpleNamespace(call=validate_delete_args))
    table.delete_resource_tag(client, "foo")


def test_delete_global_tables() -> None:
    table, _ = round_trip_for(AwsDynamoDbGlobalTable)

    def validate_delete_args(**kwargs: Any) -> Any:
        assert kwargs["action"] == "delete-table"
        assert kwargs["TableName"] == table.name

    client = cast(AwsClient, SimpleNamespace(call=validate_delete_args))
    table.delete_resource(client, Graph())



from concurrent.futures import ThreadPoolExecutor
from queue import Queue
from typing import Iterator

from pytest import fixture

from resoto_plugin_aws.collector import AwsAccountCollector
from resoto_plugin_aws.configuration import AwsConfig
from resoto_plugin_aws.aws_client import AwsClient
from resoto_plugin_aws.resource.base import AwsAccount, AwsRegion, GraphBuilder, ExecutorQueue
from resotolib.baseresources import Cloud
from resotolib.core.actions import CoreFeedback
from resotolib.graph import Graph
from test.resources import BotoFileBasedSession


@fixture
def aws_config() -> AwsConfig:
    config = AwsConfig()
    config.sessions().session_class_factory = BotoFileBasedSession
    return config


@fixture
def aws_client(aws_config: AwsConfig) -> AwsClient:
    return AwsClient(aws_config, "test", region="us-east-1")


@fixture
def builder(aws_client: AwsClient, no_feedback: CoreFeedback) -> Iterator[GraphBuilder]:
    with ThreadPoolExecutor(1) as executor:
        queue = ExecutorQueue(executor, lambda _: 1, "dummy")
        yield GraphBuilder(
            Graph(), Cloud(id="aws"), AwsAccount(id="test"), AwsRegion(id="us-east-1"), aws_client, queue, no_feedback
        )


@fixture
def no_feedback() -> CoreFeedback:
    return CoreFeedback("123", "step1", "collect", Queue())


@fixture
def account_collector(aws_config: AwsConfig, no_feedback: CoreFeedback) -> AwsAccountCollector:
    account = AwsAccount(id="test")
    return AwsAccountCollector(aws_config, Cloud(id="aws"), account, ["us-east-1"], no_feedback)



from typing import cast, Any
from types import SimpleNamespace

from resotolib.graph import Graph
from test.resources import round_trip_for

from resoto_plugin_aws.aws_client import AwsClient
from resoto_plugin_aws.resource.kms import AwsKmsKey


def test_keys() -> None:
    first, builder = round_trip_for(AwsKmsKey)
    assert first.kms_key_rotation_enabled is True
    assert len(builder.resources_of(AwsKmsKey)) == 2
    assert len(first.tags) == 1


def test_tagging_keys() -> None:
    key, _ = round_trip_for(AwsKmsKey)

    def validate_update_args(**kwargs: Any) -> Any:
        if kwargs["action"] == "tag-resource":
            assert kwargs["KeyId"] == key.id
            assert kwargs["Tags"] == [{"TagKey": "foo", "TagValue": "bar"}]

    def validate_delete_args(**kwargs: Any) -> Any:
        if kwargs["action"] == "untag-resource":
            assert kwargs["KeyId"] == key.id
            assert kwargs["TagKeys"] == ["foo"]

    client = cast(AwsClient, SimpleNamespace(call=validate_update_args))
    key.update_resource_tag(client, "foo", "bar")

    client = cast(AwsClient, SimpleNamespace(call=validate_delete_args))
    key.delete_resource_tag(client, "foo")


def test_disable_keys() -> None:
    key, _ = round_trip_for(AwsKmsKey)

    def validate_delete_args(**kwargs: Any) -> None:
        assert kwargs["action"] == "disable-key"
        assert kwargs["KeyId"] == key.id

    client = cast(AwsClient, SimpleNamespace(call=validate_delete_args))
    key.delete_resource(client, Graph())


def test_delete_keys() -> None:
    key, _ = round_trip_for(AwsKmsKey)

    def validate_delete_args(**kwargs: Any) -> None:
        assert kwargs["action"] == "disable-key"
        assert kwargs["KeyId"] == key.id

    client = cast(AwsClient, SimpleNamespace(call=validate_delete_args))
    key.delete_resource(client, Graph())


def test_normalise_keys() -> None:
    assert AwsKmsKey.normalise_id("arn:aws:kms:us-west-2:test:key/kms-1") == "kms-1"
    assert AwsKmsKey.normalise_id("kms-1") == "kms-1"



from datetime import datetime
from typing import ClassVar, Dict, List, Optional, Type

from attrs import define, field

from resoto_plugin_aws.aws_client import AwsClient
from resoto_plugin_aws.resource.base import AwsApiSpec, AwsResource, GraphBuilder
from resoto_plugin_aws.resource.kms import AwsKmsKey
from resotolib.baseresources import ModelReference
from resotolib.graph import Graph
from resotolib.json_bender import F, Bender, S, AsInt, AsBool, Bend, ParseJson
from resotolib.types import Json
from resotolib.utils import utc_str

service_name = "sqs"


@define(eq=False, slots=False)
class AwsSqsRedrivePolicy:
    kind: ClassVar[str] = "aws_sqs_redrive_policy"
    mapping: ClassVar[Dict[str, Bender]] = {
        "dead_letter_target_arn": S("deadLetterTargetArn"),
        "max_receive_count": S("maxReceiveCount"),
    }
    dead_letter_target_arn: Optional[str] = None
    max_receive_count: Optional[int] = None


@define(eq=False, slots=False)
class AwsSqsQueue(AwsResource):
    kind: ClassVar[str] = "aws_sqs_queue"
    api_spec: ClassVar[AwsApiSpec] = AwsApiSpec(service_name, "list-queues", "QueueUrls")
    reference_kinds: ClassVar[ModelReference] = {
        "successors": {"default": ["aws_kms_key"]},
        "predecessors": {"delete": ["aws_kms_key"]},
    }
    mapping: ClassVar[Dict[str, Bender]] = {
        "id": S("QueueName"),
        "name": S("QueueName"),
        "ctime": S("CreatedTimestamp") >> AsInt() >> F(lambda x: utc_str(datetime.utcfromtimestamp(x))),
        "mtime": S("LastModifiedTimestamp") >> AsInt() >> F(lambda x: utc_str(datetime.utcfromtimestamp(x))),
        "arn": S("QueueArn"),
        "sqs_queue_url": S("QueueUrl"),
        "sqs_approximate_number_of_messages": S("ApproximateNumberOfMessages") >> AsInt(),
        "sqs_approximate_number_of_messages_not_visible": S("ApproximateNumberOfMessagesNotVisible") >> AsInt(),
        "sqs_approximate_number_of_messages_delayed": S("ApproximateNumberOfMessagesDelayed") >> AsInt(),
        "sqs_policy": S("Policy") >> ParseJson(keys_to_snake=True),
        "sqs_redrive_policy": S("RedrivePolicy") >> ParseJson() >> Bend(AwsSqsRedrivePolicy.mapping),
        "sqs_fifo_queue": S("FifoQueue"),
        "sqs_content_based_deduplication": S("ContentBasedDeduplication") >> AsBool(),
        "sqs_kms_master_key_id": S("KmsMasterKeyId"),
        "sqs_kms_data_key_reuse_period_seconds": S("KmsDataKeyReusePeriodSeconds") >> AsInt(),
        "sqs_deduplication_scope": S("DeduplicationScope"),
        "sqs_fifo_throughput_limit": S("FifoThroughputLimit"),
        "sqs_redrive_allow_policy": S("RedriveAllowPolicy") >> ParseJson() >> S("redrivePermission"),
        "sqs_visibility_timeout": S("VisibilityTimeout") >> AsInt(),
        "sqs_maximum_message_size": S("MaximumMessageSize") >> AsInt(),
        "sqs_message_retention_period": S("MessageRetentionPeriod") >> AsInt(),
        "sqs_delay_seconds": S("DelaySeconds") >> AsInt(),
        "sqs_receive_message_wait_time_seconds": S("ReceiveMessageWaitTimeSeconds") >> AsInt(),
        "sqs_managed_sse_enabled": S("SqsManagedSseEnabled") >> AsBool(),
    }
    sqs_queue_url: Optional[str] = field(default=None)
    sqs_approximate_number_of_messages: Optional[int] = field(default=None)
    sqs_approximate_number_of_messages_not_visible: Optional[int] = field(default=None)
    sqs_approximate_number_of_messages_delayed: Optional[int] = field(default=None)
    sqs_policy: Optional[Json] = field(default=None)
    sqs_redrive_policy: Optional[AwsSqsRedrivePolicy] = field(default=None)
    sqs_fifo_queue: Optional[bool] = field(default=None)
    sqs_content_based_deduplication: Optional[bool] = field(default=None)
    sqs_kms_master_key_id: Optional[str] = field(default=None)
    sqs_kms_data_key_reuse_period_seconds: Optional[int] = field(default=None)
    sqs_deduplication_scope: Optional[str] = field(default=None)
    sqs_fifo_throughput_limit: Optional[str] = field(default=None)
    sqs_redrive_allow_policy: Optional[str] = field(default=None)
    sqs_visibility_timeout: Optional[int] = field(default=None)
    sqs_maximum_message_size: Optional[int] = field(default=None)
    sqs_message_retention_period: Optional[int] = field(default=None)
    sqs_delay_seconds: Optional[int] = field(default=None)
    sqs_receive_message_wait_time_seconds: Optional[int] = field(default=None)
    sqs_managed_sse_enabled: Optional[bool] = field(default=None)

    @classmethod
    def called_collect_apis(cls) -> List[AwsApiSpec]:
        return [
            cls.api_spec,
            AwsApiSpec(service_name, "get-queue-attributes"),
            AwsApiSpec(service_name, "list-queue-tags"),
        ]

    @classmethod
    def collect(cls: Type[AwsResource], json: List[Json], builder: GraphBuilder) -> None:
        def add_instance(queue_url: str) -> None:
            queue_attributes = builder.client.get(
                service_name, "get-queue-attributes", "Attributes", QueueUrl=queue_url, AttributeNames=["All"]
            )
            if queue_attributes is not None:
                queue_attributes["QueueUrl"] = queue_url
                queue_attributes["QueueName"] = queue_url.rsplit("/", 1)[-1]
                instance = cls.from_api(queue_attributes)
                builder.add_node(instance)
                builder.submit_work(service_name, add_tags, instance)

        def add_tags(queue: AwsSqsQueue) -> None:
            tags = builder.client.get(service_name, "list-queue-tags", result_name="Tags", QueueUrl=queue.sqs_queue_url)
            if tags:
                queue.tags = tags

        for queue_url in json:
            if isinstance(queue_url, str):
                add_instance(queue_url)

    def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
        if self.sqs_kms_master_key_id:
            builder.dependant_node(
                self,
                clazz=AwsKmsKey,
                id=self.sqs_kms_master_key_id,
            )

    def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
        client.call(
            aws_service=service_name,
            action="tag-queue",
            result_name=None,
            QueueUrl=self.sqs_queue_url,
            Tags={key: value},
        )
        return True

    def delete_resource_tag(self, client: AwsClient, key: str) -> bool:
        client.call(
            aws_service=service_name, action="untag-queue", result_name=None, QueueUrl=self.sqs_queue_url, TagKeys=[key]
        )
        return True

    def delete_resource(self, client: AwsClient, graph: Graph) -> bool:
        client.call(aws_service=service_name, action="delete-queue", result_name=None, QueueUrl=self.sqs_queue_url)
        return True

    @classmethod
    def called_mutator_apis(cls) -> List[AwsApiSpec]:
        return [
            AwsApiSpec(service_name, "tag-queue"),
            AwsApiSpec(service_name, "untag-queue"),
            AwsApiSpec(service_name, "delete-queue"),
        ]


resources: List[Type[AwsResource]] = [AwsSqsQueue]



from typing import Optional, ClassVar, Dict, List, Type

import math
from attr import field, define

from resoto_plugin_aws.aws_client import AwsClient
from resoto_plugin_aws.resource.base import AwsApiSpec, GraphBuilder, AwsResource
from resoto_plugin_aws.resource.kms import AwsKmsKey
from resoto_plugin_aws.utils import ToDict
from resotolib.baseresources import ModelReference, BaseNetworkShare
from resotolib.graph import Graph
from resotolib.json_bender import Bender, S, F, Bend
from resotolib.types import Json

service_name = "efs"


class EfsTaggable:
    def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
        client.call(
            aws_service=service_name,
            action="tag-resource",
            result_name=None,
            resourceId=self.id,  # type: ignore
            tags={key: value},
        )
        return True

    def delete_resource_tag(self, client: AwsClient, key: str) -> bool:
        client.call(
            aws_service=service_name,
            action="untag-resource",
            result_name=None,
            resourceId=self.id,  # type: ignore
            tagKeys=[key],
        )
        return True

    @classmethod
    def called_mutator_apis(cls) -> List[AwsApiSpec]:
        return [
            AwsApiSpec(service_name, "tag-resource", override_iam_permission="elasticfilesystem:TagResource"),
            AwsApiSpec(service_name, "untag-resource", override_iam_permission="elasticfilesystem:UntagResource"),
        ]


@define(eq=False, slots=False)
class AwsEfsMountTarget(AwsResource):
    kind: ClassVar[str] = "aws_efs_mount_target"
    mapping: ClassVar[Dict[str, Bender]] = {
        "id": S("MountTargetId"),
        "owner_id": S("OwnerId"),
        "life_cycle_state": S("LifeCycleState"),
        "ip_address": S("IpAddress"),
        "availability_zone_name": S("AvailabilityZoneName"),
    }
    reference_kinds: ClassVar[ModelReference] = {"predecessors": {"default": ["aws_ec2_network_interface"]}}
    owner_id: Optional[str] = field(default=None)
    life_cycle_state: Optional[str] = field(default=None)
    ip_address: Optional[str] = field(default=None)
    availability_zone_name: Optional[str] = field(default=None)

    def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
        if nic_id := source.get("NetworkInterfaceId"):
            builder.dependant_node(self, reverse=True, kind="aws_ec2_network_interface", id=nic_id)


@define(eq=False, slots=False)
class AwsEfsFileSystem(EfsTaggable, AwsResource, BaseNetworkShare):
    kind: ClassVar[str] = "aws_efs_file_system"
    api_spec: ClassVar[AwsApiSpec] = AwsApiSpec(
        service_name,
        "describe-file-systems",
        "FileSystems",
        override_iam_permission="elasticfilesystem:DescribeFileSystems",
    )
    mapping: ClassVar[Dict[str, Bender]] = {
        "id": S("FileSystemId"),
        "tags": S("Tags", default=[]) >> ToDict(),
        "name": S("Name"),
        "ctime": S("CreationTime"),
        "owner_id": S("OwnerId"),
        "creation_token": S("CreationToken"),
        "arn": S("FileSystemArn"),
        "share_status": S("LifeCycleState"),
        "share_size": S("SizeInBytes", "Value") >> F(lambda x: math.ceil(x / 1024**3)),
        "share_encrypted": S("Encrypted"),
        "share_throughput": S("ProvisionedThroughputInMibps") >> F(lambda x: x * 1024**2),
        "provisioned_throughput_in_mibps": S("ProvisionedThroughputInMibps"),
        "number_of_mount_targets": S("NumberOfMountTargets"),
        "performance_mode": S("PerformanceMode"),
        "throughput_mode": S("ThroughputMode"),
        "availability_zone_name": S("AvailabilityZoneName"),
    }
    reference_kinds: ClassVar[ModelReference] = {"successors": {"default": ["aws_kms_key"]}}
    owner_id: Optional[str] = field(default=None)
    creation_token: Optional[str] = field(default=None)
    number_of_mount_targets: Optional[int] = field(default=None)
    performance_mode: Optional[str] = field(default=None)
    throughput_mode: Optional[str] = field(default=None)
    provisioned_throughput_in_mibps: Optional[float] = field(default=None)
    availability_zone_name: Optional[str] = field(default=None)

    @classmethod
    def called_collect_apis(cls) -> List[AwsApiSpec]:
        return [
            cls.api_spec,
            AwsApiSpec(
                service_name, "describe-mount-targets", override_iam_permission="elasticfilesystem:DescribeMountTargets"
            ),
        ]

    @classmethod
    def collect(cls: Type[AwsResource], json: List[Json], builder: GraphBuilder) -> None:
        def collect_mount_points(fs: AwsEfsFileSystem) -> None:
            for mt_raw in builder.client.list(
                service_name, "describe-mount-targets", "MountTargets", FileSystemId=fs.id
            ):
                mt = AwsEfsMountTarget.from_api(mt_raw)
                builder.add_node(mt, mt_raw)
                builder.add_edge(fs, node=mt)

        for js in json:
            instance = cls.from_api(js)
            builder.add_node(instance, js)
            builder.submit_work(service_name, collect_mount_points, instance)

    def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
        if kms_key_id := source.get("KmsKeyId"):
            builder.dependant_node(from_node=self, clazz=AwsKmsKey, id=AwsKmsKey.normalise_id(kms_key_id))

    def delete_resource(self, client: AwsClient, graph: Graph) -> bool:
        client.call(service_name, "delete-file-system", FileSystemId=self.id)
        return True

    @classmethod
    def called_mutator_apis(cls) -> List[AwsApiSpec]:
        return [
            AwsApiSpec(service_name, "delete-file-system", override_iam_permission="elasticfilesystem:DeleteFileSystem")
        ]


@define(eq=False, slots=False)
class AwsEfsPosixUser:
    kind: ClassVar[str] = "aws_efs_posix_user"
    mapping: ClassVar[Dict[str, Bender]] = {
        "uid": S("Uid"),
        "gid": S("Gid"),
        "secondary_gids": S("SecondaryGids", default=[]),
    }
    uid: Optional[int] = field(default=None)
    gid: Optional[int] = field(default=None)
    secondary_gids: List[int] = field(factory=list)


@define(eq=False, slots=False)
class AwsEfsCreationInfo:
    kind: ClassVar[str] = "aws_efs_creation_info"
    mapping: ClassVar[Dict[str, Bender]] = {
        "owner_uid": S("OwnerUid"),
        "owner_gid": S("OwnerGid"),
        "permissions": S("Permissions"),
    }
    owner_uid: Optional[int] = field(default=None)
    owner_gid: Optional[int] = field(default=None)
    permissions: Optional[str] = field(default=None)


@define(eq=False, slots=False)
class AwsEfsRootDirectory:
    kind: ClassVar[str] = "aws_efs_root_directory"
    mapping: ClassVar[Dict[str, Bender]] = {
        "path": S("Path"),
        "creation_info": S("CreationInfo") >> Bend(AwsEfsCreationInfo.mapping),
    }
    path: Optional[str] = field(default=None)
    creation_info: Optional[AwsEfsCreationInfo] = field(default=None)


@define(eq=False, slots=False)
class AwsEfsAccessPoint(AwsResource, EfsTaggable):
    kind: ClassVar[str] = "aws_efs_access_point"
    api_spec: ClassVar[AwsApiSpec] = AwsApiSpec(
        service_name,
        "describe-access-points",
        "AccessPoints",
        override_iam_permission="elasticfilesystem:DescribeAccessPoints",
    )
    mapping: ClassVar[Dict[str, Bender]] = {
        "id": S("AccessPointId"),
        "tags": S("Tags", default=[]) >> ToDict(),
        "name": S("Name"),
        "client_token": S("ClientToken"),
        "arn": S("AccessPointArn"),
        "posix_user": S("PosixUser") >> Bend(AwsEfsPosixUser.mapping),
        "root_directory": S("RootDirectory") >> Bend(AwsEfsRootDirectory.mapping),
        "owner_id": S("OwnerId"),
        "life_cycle_state": S("LifeCycleState"),
    }
    reference_kinds: ClassVar[ModelReference] = {"predecessors": {"default": ["aws_efs_file_system"]}}
    client_token: Optional[str] = field(default=None)
    posix_user: Optional[AwsEfsPosixUser] = field(default=None)
    root_directory: Optional[AwsEfsRootDirectory] = field(default=None)
    owner_id: Optional[str] = field(default=None)
    life_cycle_state: Optional[str] = field(default=None)

    def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
        if fs_id := source.get("FileSystemId"):
            builder.dependant_node(
                from_node=self, reverse=True, delete_same_as_default=True, clazz=AwsEfsFileSystem, id=fs_id
            )


resources: List[Type[AwsResource]] = [AwsEfsFileSystem, AwsEfsAccessPoint]



from datetime import datetime, timezone
from textwrap import dedent
from types import SimpleNamespace
from typing import Any, cast

from resoto_plugin_aws.aws_client import AwsClient
from resoto_plugin_aws.resource.iam import (
    AwsIamPolicy,
    AwsIamGroup,
    AwsIamServerCertificate,
    AwsIamRole,
    AwsIamUser,
    AwsIamAccessKey,
    AwsIamInstanceProfile,
    CredentialReportLine,
)
from resotolib.graph import Graph
from test.resources import round_trip_for


def test_credentials_report() -> None:
    csv = dedent(
        """
        user,arn,user_creation_time,password_enabled,password_last_used,password_last_changed,password_next_rotation,mfa_active,access_key_1_active,access_key_1_last_rotated,access_key_1_last_used_date,access_key_1_last_used_region,access_key_1_last_used_service,access_key_2_active,access_key_2_last_rotated,access_key_2_last_used_date,access_key_2_last_used_region,access_key_2_last_used_service,cert_1_active,cert_1_last_rotated,cert_2_active,cert_2_last_rotated
        a,arn:aws:iam::test:user/a,2021-05-06T08:23:17+00:00,true,2021-12-08T09:34:46+00:00,2021-05-06T08:30:03+00:00,N/A,true,true,2021-05-06T08:23:18+00:00,2023-01-25T10:11:00+00:00,us-east-1,iam,false,N/A,N/A,N/A,N/A,false,N/A,false,N/A
        b,arn:aws:iam::test:user/b,2022-05-06T08:23:17+00:00,false,2022-12-08T09:34:46+00:00,2022-05-06T08:30:03+00:00,N/A,false,false,2022-05-06T08:23:18+00:00,2023-01-25T10:11:00+00:00,eu-central-1,s3,true,2023-01-25T10:11:00+00:00,2023-01-25T10:11:00+00:00,eu-central-3,iam,false,N/A,false,N/A
        c,arn:aws:iam::test:user/c,2022-05-06T08:23:17+00:00,false,no_information,2022-05-06T08:30:03+00:00,N/A,false,false,2022-05-06T08:23:18+00:00,2023-01-25T10:11:00+00:00,eu-central-1,s3,true,2023-01-25T10:11:00+00:00,2023-01-25T10:11:00+00:00,eu-central-3,iam,false,N/A,false,N/A
        """
    ).strip()
    lines = CredentialReportLine.from_str(csv)
    assert len(lines) == 3
    assert lines["a"].password_enabled() is True
    assert lines["a"].password_last_used() == datetime(2021, 12, 8, 9, 34, 46, tzinfo=timezone.utc)
    assert [a.access_key_last_used.service_name for a in lines["a"].access_keys() if a.access_key_last_used] == ["iam"]
    assert lines["b"].password_enabled() is False
    assert lines["b"].password_last_used() == datetime(2022, 12, 8, 9, 34, 46, tzinfo=timezone.utc)
    assert [b.access_key_last_used.service_name for b in lines["b"].access_keys() if b.access_key_last_used] == [
        "s3",
        "iam",
    ]


def test_server_certificates() -> None:
    round_trip_for(AwsIamServerCertificate, "dns_names", "sha1_fingerprint")


def test_instance_profiles() -> None:
    round_trip_for(AwsIamInstanceProfile)


def test_user_roles_groups_policies_keys() -> None:
    _, builder = round_trip_for(AwsIamUser)

    # users ------------
    assert len(builder.resources_of(AwsIamUser)) == 3
    assert (test_user := builder.node(clazz=AwsIamUser, name="test_user")) is not None
    assert test_user.atime is not None
    assert [p.policy_name for p in test_user.user_policies] == ["stsAssumeRole"]

    # keys ------------
    assert len(builder.resources_of(AwsIamAccessKey)) == 2
    assert (ak_test := builder.node(clazz=AwsIamAccessKey, id="ak_test")) is not None
    assert len(builder.graph.nodes) == 13
    # make sure access keys are created and connected as part of the user
    assert builder.graph.has_edge(test_user, ak_test)

    # groups ------------
    assert len(builder.resources_of(AwsIamGroup)) == 1
    assert (group := builder.node(clazz=AwsIamGroup, name="test_group")) is not None
    assert [p.policy_name for p in group.group_policies] == ["stsAssumeRole"]

    # policies ------------
    assert len(builder.resources_of(AwsIamPolicy)) == 4
    assert (policy := builder.node(clazz=AwsIamPolicy, name="master-elb")) is not None
    assert policy.policy_attachment_count == 1
    assert policy.policy_document is not None

    # roles
    assert len(builder.resources_of(AwsIamRole)) == 2
    assert (role := builder.node(clazz=AwsIamRole, name="role1")) is not None
    assert len(role.role_policies) == 1


def test_server_certificate_deletion() -> None:
    res, _ = round_trip_for(AwsIamServerCertificate, "dns_names", "sha1_fingerprint")

    def validate_update_args(**kwargs: Any) -> None:
        assert kwargs["action"] == "delete-server-certificate"
        assert kwargs["ServerCertificateName"] == res.name

    client = cast(AwsClient, SimpleNamespace(call=validate_update_args))
    res.delete_resource(client, Graph())


def test_aws_iam_policy_deletion() -> None:
    _, builder = round_trip_for(AwsIamUser)
    res = builder.resources_of(AwsIamPolicy)[0]

    def validate_update_args(**kwargs: Any) -> None:
        assert kwargs["action"] == "delete-policy"
        assert kwargs["PolicyArn"] == res.arn

    client = cast(AwsClient, SimpleNamespace(call=validate_update_args))
    res.delete_resource(client, Graph())


def test_aws_iam_group_deletion() -> None:
    _, builder = round_trip_for(AwsIamUser)
    res = builder.resources_of(AwsIamGroup)[0]

    def validate_update_args(**kwargs: Any) -> None:
        assert kwargs["action"] == "delete-group"
        assert kwargs["GroupName"] == res.name

    client = cast(AwsClient, SimpleNamespace(call=validate_update_args))
    res.delete_resource(client, Graph())


def test_aws_iam_role_deletion() -> None:
    _, builder = round_trip_for(AwsIamUser)
    res = builder.resources_of(AwsIamRole)[0]

    def validate_update_args(**kwargs: Any) -> None:
        assert kwargs["action"] == "delete-role"
        assert kwargs["RoleName"] == res.name

    client = cast(AwsClient, SimpleNamespace(call=validate_update_args))
    res.delete_resource(client, Graph())


def test_aws_iam_user_deletion() -> None:
    _, builder = round_trip_for(AwsIamUser)
    res = builder.resources_of(AwsIamUser)[0]

    def validate_update_args(**kwargs: Any) -> None:
        assert kwargs["action"] == "delete-user"
        assert kwargs["UserName"] == res.name

    client = cast(AwsClient, SimpleNamespace(call=validate_update_args))
    res.delete_resource(client, Graph())


def test_aws_iam_instance_profile_deletion() -> None:
    res, _ = round_trip_for(AwsIamInstanceProfile)

    def validate_update_args(**kwargs: Any) -> None:
        assert kwargs["action"] == "delete-instance-profile"
        assert kwargs["InstanceProfileName"] == res.name

    client = cast(AwsClient, SimpleNamespace(call=validate_update_args))
    res.delete_resource(client, Graph())


def test_tagging() -> None:
    res, _ = round_trip_for(AwsIamServerCertificate, "dns_names", "sha1_fingerprint")

    def validate_update_args(**kwargs: Any) -> None:
        assert kwargs["action"] == "tag-server-certificate"
        assert kwargs["Tags"] == [{"Key": "foo", "Value": "bar"}]
        assert kwargs["ServerCertificateName"] == res.name

    def validate_delete_args(**kwargs: Any) -> None:
        assert kwargs["action"] == "untag-server-certificate"
        assert kwargs["TagKeys"] == ["foo"]
        assert kwargs["ServerCertificateName"] == res.name

    client = cast(AwsClient, SimpleNamespace(call=validate_update_args))
    res.update_resource_tag(client, "foo", "bar")

    client = cast(AwsClient, SimpleNamespace(call=validate_delete_args))
    res.delete_resource_tag(client, "foo")



from typing import ClassVar, Dict, List, Optional, Type
from attrs import define, field
from resoto_plugin_aws.aws_client import AwsClient
from resoto_plugin_aws.resource.base import AwsApiSpec, AwsResource, GraphBuilder
from resoto_plugin_aws.resource.iam import AwsIamRole
from resoto_plugin_aws.resource.kms import AwsKmsKey
from resoto_plugin_aws.utils import ToDict
from resotolib.baseresources import EdgeType, ModelReference
from resotolib.graph import Graph
from resotolib.json_bender import F, Bender, S, bend
from resotolib.types import Json

service_name = "sns"


@define(eq=False, slots=False)
class AwsSnsTopic(AwsResource):
    kind: ClassVar[str] = "aws_sns_topic"
    api_spec: ClassVar[AwsApiSpec] = AwsApiSpec(service_name, "list-topics", "Topics")
    reference_kinds: ClassVar[ModelReference] = {
        "predecessors": {
            "delete": ["aws_kms_key"],
        },
        "successors": {"default": ["aws_kms_key"]},
    }
    mapping: ClassVar[Dict[str, Bender]] = {
        "id": S("TopicArn"),
        "name": S("DisplayName"),
        "arn": S("TopicArn"),
        "topic_subscriptions_confirmed": S("SubscriptionsConfirmed") >> F(lambda x: int(x)),
        "topic_subscriptions_deleted": S("SubscriptionsDeleted") >> F(lambda x: int(x)),
        "topic_subscriptions_pending": S("SubscriptionsPending") >> F(lambda x: int(x)),
        "topic_policy": S("Policy"),
        "topic_delivery_policy": S("DeliveryPolicy"),
        "topic_effective_delivery_policy": S("EffectiveDeliveryPolicy"),
        "topic_owner": S("Owner"),
        "topic_kms_master_key_id": S("KmsMasterKeyId"),
        "topic_fifo_topic": S("FifoTopic") >> F(lambda x: x == "true"),
        "topic_content_based_deduplication": S("ContentBasedDeduplication") >> F(lambda x: x == "true"),
    }
    topic_subscriptions_confirmed: Optional[int] = field(default=None)
    topic_subscriptions_deleted: Optional[int] = field(default=None)
    topic_subscriptions_pending: Optional[int] = field(default=None)
    topic_policy: Optional[str] = field(default=None)
    topic_delivery_policy: Optional[str] = field(default=None)
    topic_effective_delivery_policy: Optional[str] = field(default=None)
    topic_owner: Optional[str] = field(default=None)
    topic_kms_master_key_id: Optional[str] = field(default=None)
    topic_fifo_topic: Optional[bool] = field(default=None)
    topic_content_based_deduplication: Optional[bool] = field(default=None)

    @classmethod
    def called_collect_apis(cls) -> List[AwsApiSpec]:
        return [
            cls.api_spec,
            AwsApiSpec(service_name, "get-topic-attributes"),
            AwsApiSpec(service_name, "list-tags-for-resource"),
        ]

    @classmethod
    def collect(cls: Type[AwsResource], json: List[Json], builder: GraphBuilder) -> None:
        def add_tags(topic: AwsSnsTopic) -> None:
            tags = builder.client.list(
                service_name, "list-tags-for-resource", result_name="Tags", ResourceArn=topic.arn
            )
            if tags:
                topic.tags = bend(ToDict(), tags)

        for entry in json:
            topic = builder.client.get(
                service_name, "get-topic-attributes", TopicArn=entry["TopicArn"], result_name="Attributes"
            )
            if topic:
                topic_instance = cls.from_api(topic)
                builder.add_node(topic_instance, topic)
                builder.submit_work(service_name, add_tags, topic_instance)

    def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
        if self.topic_kms_master_key_id:
            builder.dependant_node(
                self,
                clazz=AwsKmsKey,
                id=AwsKmsKey.normalise_id(self.topic_kms_master_key_id),
            )

    def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
        client.call(
            aws_service=service_name,
            action="tag-resource",
            result_name=None,
            ResourceArn=self.arn,
            Tags=[{"Key": key, "Value": value}],
        )
        return True

    def delete_resource_tag(self, client: AwsClient, key: str) -> bool:
        client.call(
            aws_service=service_name, action="untag-resource", result_name=None, ResourceArn=self.arn, TagKeys=[key]
        )
        return True

    def delete_resource(self, client: AwsClient, graph: Graph) -> bool:
        client.call(aws_service=service_name, action="delete-topic", result_name=None, TopicArn=self.arn)
        return True

    @classmethod
    def called_mutator_apis(cls) -> List[AwsApiSpec]:
        return [
            AwsApiSpec(service_name, "tag-resource"),
            AwsApiSpec(service_name, "untag-resource"),
            AwsApiSpec(service_name, "delete-topic"),
        ]


@define(eq=False, slots=False)
class AwsSnsSubscription(AwsResource):
    kind: ClassVar[str] = "aws_sns_subscription"
    api_spec: ClassVar[AwsApiSpec] = AwsApiSpec(service_name, "list-subscriptions", "Subscriptions")
    reference_kinds: ClassVar[ModelReference] = {
        "predecessors": {"default": ["aws_sns_topic", "aws_iam_role"], "delete": ["aws_iam_role"]},
    }
    mapping: ClassVar[Dict[str, Bender]] = {
        "id": S("SubscriptionArn"),
        "name": S("SubscriptionArn"),
        "arn": S("SubscriptionArn"),
        "subscription_confirmation_was_authenticated": S("ConfirmationWasAuthenticated") >> F(lambda x: x == "true"),
        "subscription_delivery_policy": S("DeliveryPolicy"),
        "subscription_effective_delivery_policy": S("EffectiveDeliveryPolicy"),
        "subscription_filter_policy": S("FilterPolicy"),
        "subscription_owner": S("Owner"),
        "subscription_pending_confirmation": S("PendingConfirmation") >> F(lambda x: x == "true"),
        "subscription_raw_message_delivery": S("RawMessageDelivery") >> F(lambda x: x == "true"),
        "subscription_redrive_policy": S("RedrivePolicy"),
        "subscription_topic_arn": S("TopicArn"),
        "subscription_role_arn": S("SubscriptionRoleArn"),
    }
    subscription_confirmation_was_authenticated: Optional[bool] = field(default=None)
    subscription_delivery_policy: Optional[str] = field(default=None)
    subscription_effective_delivery_policy: Optional[str] = field(default=None)
    subscription_filter_policy: Optional[str] = field(default=None)
    subscription_owner: Optional[str] = field(default=None)
    subscription_pending_confirmation: Optional[bool] = field(default=None)
    subscription_raw_message_delivery: Optional[bool] = field(default=None)
    subscription_redrive_policy: Optional[str] = field(default=None)
    subscription_topic_arn: Optional[str] = field(default=None)
    subscription_role_arn: Optional[str] = field(default=None)

    @classmethod
    def called_collect_apis(cls) -> List[AwsApiSpec]:
        return [
            cls.api_spec,
            AwsApiSpec(service_name, "get-subscription-attributes"),
        ]

    @classmethod
    def collect(cls: Type[AwsResource], json: List[Json], builder: GraphBuilder) -> None:
        for entry in json:
            subscription = builder.client.get(
                service_name,
                "get-subscription-attributes",
                SubscriptionArn=entry["SubscriptionArn"],
                result_name="Attributes",
                expected_errors=["InvalidParameter"],
            )
            if subscription:
                subscription_instance = cls.from_api(subscription)
                builder.add_node(subscription_instance, subscription)

    def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
        if self.subscription_topic_arn:
            builder.add_edge(
                self,
                reverse=True,
                clazz=AwsSnsTopic,
                arn=self.subscription_topic_arn,
            )
        if self.subscription_role_arn:
            builder.dependant_node(
                self, reverse=True, delete_same_as_default=True, clazz=AwsIamRole, arn=self.subscription_role_arn
            )

    def delete_resource(self, client: AwsClient, graph: Graph) -> bool:
        client.call(aws_service=service_name, action="unsubscribe", result_name=None, SubscriptionArn=self.arn)
        return True

    @classmethod
    def called_mutator_apis(cls) -> List[AwsApiSpec]:
        return [AwsApiSpec(service_name, "unsubscribe")]


@define(eq=False, slots=False)
class AwsSnsEndpoint(AwsResource):
    # collection of endpoint resources happens in AwsSnsPlatformApplication.collect()
    kind: ClassVar[str] = "aws_sns_endpoint"
    mapping: ClassVar[Dict[str, Bender]] = {
        "id": S("Arn"),
        "arn": S("Arn"),
        "endpoint_enabled": S("Enabled") >> F(lambda x: x == "true"),
        "endpoint_token": S("Token"),
    }
    endpoint_enabled: Optional[bool] = field(default=None)
    endpoint_token: Optional[str] = field(default=None)

    def delete_resource(self, client: AwsClient, graph: Graph) -> bool:
        client.call(aws_service=service_name, action="delete-endpoint", result_name=None, EndpointArn=self.arn)
        return True

    @classmethod
    def called_mutator_apis(cls) -> List[AwsApiSpec]:
        return [AwsApiSpec(service_name, "delete-endpoint")]


@define(eq=False, slots=False)
class AwsSnsPlatformApplication(AwsResource):
    kind: ClassVar[str] = "aws_sns_platform_application"
    api_spec: ClassVar[AwsApiSpec] = AwsApiSpec(
        service_name, "list-platform-applications", "PlatformApplications", expected_errors=["InvalidAction"]
    )
    reference_kinds: ClassVar[ModelReference] = {
        "successors": {
            "default": ["aws_sns_topic", "aws_sns_endpoint"],
        },
    }
    mapping: ClassVar[Dict[str, Bender]] = {
        "id": S("Arn"),
        "arn": S("Arn"),
        "application_apple_certificate_expiry_date": S("AppleCertificateExpiryDate"),
        "application_apple_platform_team_id": S("ApplePlatformTeamID"),
        "application_apple_platform_bundle_id": S("ApplePlatformBundleID"),
        "application_event_endpoint_created": S("EventEndpointCreated"),
        "application_event_endpoint_deleted": S("EventEndpointDeleted"),
        "application_event_endpoint_updated": S("EventEndpointUpdated"),
        "application_event_endpoint_failure": S("EventDeliveryFailure"),
    }
    application_apple_certificate_expiry_date: Optional[str] = field(default=None)
    application_apple_platform_team_id: Optional[str] = field(default=None)
    application_apple_platform_bundle_id: Optional[str] = field(default=None)
    application_event_endpoint_created: Optional[str] = field(default=None)
    application_event_endpoint_deleted: Optional[str] = field(default=None)
    application_event_endpoint_updated: Optional[str] = field(default=None)
    application_event_endpoint_failure: Optional[str] = field(default=None)

    @classmethod
    def called_collect_apis(cls) -> List[AwsApiSpec]:
        return [
            cls.api_spec,
            AwsApiSpec(service_name, "get-platform-application-attributes"),
            AwsApiSpec(service_name, "list-endpoints-by-platform-application"),
        ]

    @classmethod
    def collect(cls: Type[AwsResource], json: List[Json], builder: GraphBuilder) -> None:
        for entry in json:
            app_arn = entry["PlatformApplicationArn"]
            app = builder.client.get(
                service_name,
                "get-platform-application-attributes",
                PlatformApplicationArn=app_arn,
                result_name="Attributes",
            )
            if app:
                app["Arn"] = app_arn
                app_instance = cls.from_api(app)
                builder.add_node(app_instance, app)

                endpoints = builder.client.list(
                    service_name,
                    "list-endpoints-by-platform-application",
                    PlatformApplicationArn=app_arn,
                    result_name="Endpoints",
                )
                for endpoint in endpoints:
                    attributes = endpoint["Attributes"]
                    attributes["Arn"] = endpoint["EndpointArn"]
                    endpoint_instance = AwsSnsEndpoint.from_api(attributes)
                    builder.add_node(endpoint_instance, attributes)
                    builder.add_edge(app_instance, edge_type=EdgeType.default, node=endpoint_instance)

    def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
        for topic in [
            self.application_event_endpoint_created,
            self.application_event_endpoint_deleted,
            self.application_event_endpoint_updated,
            self.application_event_endpoint_failure,
        ]:
            builder.add_edge(self, edge_type=EdgeType.default, clazz=AwsSnsTopic, arn=topic)

    def delete_resource(self, client: AwsClient, graph: Graph) -> bool:
        client.call(
            aws_service=service_name,
            action="delete-platform-application",
            result_name=None,
            PlatformApplicationArn=self.arn,
        )
        return True

    @classmethod
    def called_mutator_apis(cls) -> List[AwsApiSpec]:
        return [AwsApiSpec(service_name, "delete-platform-application")]


resources: List[Type[AwsResource]] = [AwsSnsTopic, AwsSnsSubscription, AwsSnsPlatformApplication, AwsSnsEndpoint]



from collections import defaultdict
from types import SimpleNamespace
from typing import Any, Dict, cast
from resoto_plugin_aws.aws_client import AwsClient
from resotolib.graph import Graph
from test.resources import round_trip_for
from resoto_plugin_aws.resource.apigateway import AwsApiGatewayRestApi, AwsApiGatewayDomainName


def test_rest_apis() -> None:
    api, builder = round_trip_for(AwsApiGatewayRestApi)
    assert len(builder.resources_of(AwsApiGatewayRestApi)) == 1
    assert len(api.tags) == 1
    assert api.arn == "arn:aws:apigateway:eu-central-1::/restapis/2lsd9i45ub"
    type_count: Dict[str, int] = defaultdict(int)
    for node in builder.graph.nodes:
        type_count[node.kind] += 1
    assert type_count["aws_api_gateway_deployment"] == 2
    assert type_count["aws_api_gateway_stage"] == 1
    assert type_count["aws_api_gateway_authorizer"] == 1
    assert type_count["aws_api_gateway_resource"] == 1


def test_api_tagging() -> None:
    api, builder = round_trip_for(AwsApiGatewayRestApi)

    def validate_update_args(**kwargs: Any) -> None:
        assert kwargs["action"] == "tag-resource"
        assert kwargs["resourceArn"] == api.arn
        assert kwargs["tags"] == {"foo": "bar"}

    def validate_delete_args(**kwargs: Any) -> None:
        assert kwargs["action"] == "untag-resource"
        assert kwargs["resourceArn"] == api.arn
        assert kwargs["tagKeys"] == ["foo"]

    client = cast(AwsClient, SimpleNamespace(call=validate_update_args))
    api.update_resource_tag(client, "foo", "bar")

    client = cast(AwsClient, SimpleNamespace(call=validate_delete_args))
    api.delete_resource_tag(client, "foo")


def test_delete_api() -> None:
    api, _ = round_trip_for(AwsApiGatewayRestApi)

    def validate_delete_args(**kwargs: Any) -> Any:
        assert kwargs["action"] == "delete-rest-api"
        assert kwargs["restApiId"] == api.id

    client = cast(AwsClient, SimpleNamespace(call=validate_delete_args))
    api.delete_resource(client, Graph())


def test_domain_names() -> None:
    api, builder = round_trip_for(AwsApiGatewayDomainName)
    assert len(builder.resources_of(AwsApiGatewayDomainName)) == 1
    assert len(api.tags) == 1



import json
import threading
from typing import Type, List, Tuple, Set

from networkx import DiGraph, is_directed_acyclic_graph

from resoto_plugin_aws.collector import (
    AwsAccountCollector,
    all_resources,
    called_collect_apis,
    called_mutator_apis,
)
from resoto_plugin_aws.resource.base import AwsResource, AwsApiSpec
from resotolib.core.model_export import dataclasses_to_resotocore_model
from test import account_collector, builder, aws_client, aws_config, no_feedback  # noqa: F401


def test_collect(account_collector: AwsAccountCollector) -> None:
    account_collector.collect()

    def count_kind(clazz: Type[AwsResource]) -> int:
        count = 0
        for node in account_collector.graph.nodes:
            if isinstance(node, clazz):
                count += 1
        return count

    for resource in all_resources:
        assert count_kind(resource) > 0, "No instances of {} found".format(resource.__name__)

    # make sure all threads have been joined
    assert len(threading.enumerate()) == 1
    # ensure the correct number of nodes and edges
    assert count_kind(AwsResource) == 204
    assert len(account_collector.graph.edges) == 476
    assert len(account_collector.graph.deferred_edges) == 2


def test_dependencies() -> None:
    model = dataclasses_to_resotocore_model({AwsResource})

    def for_edge_type(edge_type: str) -> DiGraph:
        graph = DiGraph()
        for kind in model:
            if (successors := kind.get("successor_kinds")) and edge_type in successors:
                for successor in successors[edge_type]:
                    graph.add_edge(kind["fqn"], successor)
        return graph

    def show_graph(graph: DiGraph) -> str:
        result = "digraph {\n"
        for start, end in graph.edges:
            result += "  " + start + "->" + end + "\n"
        result += "}"
        return result

    assert is_directed_acyclic_graph(for_edge_type("default")), show_graph(for_edge_type("default"))
    assert is_directed_acyclic_graph(for_edge_type("delete")), show_graph(for_edge_type("delete"))


def test_all_called_apis() -> None:
    def iam_statement(name: str, apis: List[AwsApiSpec]) -> Tuple[Set[str], str]:
        permissions = {api.iam_permission() for api in apis}
        doc = dict(Sid=name, Effect="Allow", Action=sorted(permissions), Resource="*")
        return permissions, json.dumps(doc, indent=2)

    collect_allow, collect_statement = iam_statement("ResotoCollectPermission", called_collect_apis())
    print("\n\n", collect_statement, "\n\n")
    assert json.loads(collect_statement)
    assert len(collect_allow) >= 74
    assert "s3:ListAllMyBuckets" in collect_allow

    mutate_allow, mutate_statement = iam_statement("ResotoMutatePermission", called_mutator_apis())
    print("\n\n", mutate_statement, "\n\n")



from typing import Tuple

from botocore.exceptions import ClientError

from resoto_plugin_aws.aws_client import AwsClient, ErrorAccumulator, is_retryable_exception
from resoto_plugin_aws.configuration import AwsConfig
from test.resources import BotoFileBasedSession, BotoErrorSession


def test_region() -> None:
    client = AwsClient(AwsConfig(), "test", role="test")
    east1 = client.for_region("us-east-1")
    assert client.account_id == east1.account_id
    assert client.role == east1.role
    assert client.config == east1.config
    assert client.region is None
    assert east1.region == "us-east-1"


def test_call() -> None:
    config = AwsConfig(access_key_id="foo", secret_access_key="bar")
    config.sessions().session_class_factory = BotoFileBasedSession
    client = AwsClient(config, "test")
    instances = client.list("ec2", "describe-instances", "Reservations")
    assert len(instances) == 3


def test_error_handling() -> None:
    def with_error(code: str, message: str) -> Tuple[AwsClient, ErrorAccumulator]:
        config = AwsConfig()
        config.sessions().session_class_factory = BotoErrorSession(
            ClientError({"Error": {"Code": code, "Message": message}}, "foo")
        )
        error_accumulator = ErrorAccumulator()
        return AwsClient(config, "test", error_accumulator=error_accumulator), error_accumulator

    unauthorized_client, accu_unauthorized = with_error("UnauthorizedOperation", "Err!")
    # this error is raised for the operation
    unauthorized_client.list("ec2", "foo", None)
    assert len(accu_unauthorized.regional_errors) == 1
    # we can silent this error by passing the expected error code
    assert unauthorized_client.list("ec2", "foo", None, ["UnauthorizedOperation"]) == []
    assert unauthorized_client.get("ec2", "foo", None, ["UnauthorizedOperation"]) is None
    assert len(accu_unauthorized.regional_errors) == 1

    access_denied_client, queue_access_denied = with_error("AccessDenied", "Err!")
    # this error is only logged
    assert access_denied_client.list("ec2", "foo", None) == []
    assert access_denied_client.get("ec2", "foo", None) is None
    assert len(queue_access_denied.regional_errors) == 1
    # no additional error is logged
    assert access_denied_client.list("ec2", "foo", None, ["AccessDenied"]) == []
    assert access_denied_client.get("ec2", "foo", None, ["AccessDenied"]) is None
    assert len(queue_access_denied.regional_errors) == 1

    some_error_client, queue_some_error = with_error("some_error", "Err!")
    # this error is only logged
    assert some_error_client.list("ec2", "foo", None) == []
    assert some_error_client.get("ec2", "foo", None) is None
    assert len(queue_some_error.regional_errors) == 1
    # no additional error is logged
    assert some_error_client.list("ec2", "foo", None, ["some_error"]) == []
    assert some_error_client.get("ec2", "foo", None, ["some_error"]) is None
    assert len(queue_some_error.regional_errors) == 1


def test_is_retryable() -> None:
    def check_code(code: str, expected: bool) -> None:
        assert is_retryable_exception(ClientError({"Error": {"Code": code, "Message": "eff"}}, "foo")) is expected

    check_code("ThrottlingException", True)
    check_code("RequestLimitExceeded", True)
    check_code("AccessDenied", False)
    check_code("UnauthorizedOperation", False)



from types import SimpleNamespace
from typing import Any, cast
from resoto_plugin_aws.aws_client import AwsClient
from test.resources import round_trip_for
from resoto_plugin_aws.resource.sagemaker import (
    AwsSagemakerNotebook,
    AwsSagemakerAlgorithm,
    AwsSagemakerApp,
    AwsSagemakerModel,
    AwsSagemakerDomain,
    AwsSagemakerExperiment,
    AwsSagemakerTrial,
    AwsSagemakerProject,
    AwsSagemakerCodeRepository,
    AwsSagemakerEndpoint,
    AwsSagemakerImage,
    AwsSagemakerArtifact,
    AwsSagemakerUserProfile,
    AwsSagemakerPipeline,
    AwsSagemakerWorkteam,
    AwsSagemakerAutoMLJob,
    AwsSagemakerCompilationJob,
    AwsSagemakerEdgePackagingJob,
    AwsSagemakerHyperParameterTuningJob,
    AwsSagemakerInferenceRecommendationsJob,
    AwsSagemakerLabelingJob,
    AwsSagemakerProcessingJob,
    AwsSagemakerTrainingJob,
    AwsSagemakerTransformJob,
)


def test_tagging() -> None:
    notebook, builder = round_trip_for(AwsSagemakerNotebook)

    def validate_update_args(**kwargs: Any) -> None:
        assert kwargs["action"] == "add-tags"
        assert kwargs["ResourceArn"] == notebook.arn
        assert kwargs["Tags"] == [{"Key": "foo", "Value": "bar"}]

    def validate_delete_args(**kwargs: Any) -> None:
        assert kwargs["action"] == "delete-tags"
        assert kwargs["ResourceArn"] == notebook.arn
        assert kwargs["TagKeys"] == ["foo"]

    client = cast(AwsClient, SimpleNamespace(call=validate_update_args))
    notebook.update_resource_tag(client, "foo", "bar")

    client = cast(AwsClient, SimpleNamespace(call=validate_delete_args))
    notebook.delete_resource_tag(client, "foo")


def test_notebooks() -> None:
    first, builder = round_trip_for(AwsSagemakerNotebook)
    assert len(builder.resources_of(AwsSagemakerNotebook)) == 1
    assert len(first.tags) == 1


def test_algorithms() -> None:
    first, builder = round_trip_for(AwsSagemakerAlgorithm)
    assert len(builder.resources_of(AwsSagemakerAlgorithm)) == 1


def test_apps() -> None:
    first, builder = round_trip_for(AwsSagemakerApp)
    assert len(builder.resources_of(AwsSagemakerApp)) == 1
    assert first.arn


def test_models() -> None:
    first, builder = round_trip_for(AwsSagemakerModel)
    assert len(builder.resources_of(AwsSagemakerModel)) == 1


def test_domains() -> None:
    first, builder = round_trip_for(AwsSagemakerDomain)
    assert len(builder.resources_of(AwsSagemakerDomain)) == 1


def test_experiments() -> None:
    first, builder = round_trip_for(AwsSagemakerExperiment)
    assert len(builder.resources_of(AwsSagemakerExperiment)) == 1


def test_trials() -> None:
    first, builder = round_trip_for(AwsSagemakerTrial)
    assert len(builder.resources_of(AwsSagemakerTrial)) == 1


def test_projects() -> None:
    first, builder = round_trip_for(AwsSagemakerProject)
    assert len(builder.resources_of(AwsSagemakerProject)) == 1


def test_repos() -> None:
    first, builder = round_trip_for(AwsSagemakerCodeRepository)
    assert len(builder.resources_of(AwsSagemakerCodeRepository)) == 1
    assert first.code_repository_url == "some.url"


def test_endpoint() -> None:
    first, builder = round_trip_for(AwsSagemakerEndpoint)
    assert len(builder.resources_of(AwsSagemakerEndpoint)) == 1


def test_image() -> None:
    first, builder = round_trip_for(AwsSagemakerImage)
    assert len(builder.resources_of(AwsSagemakerImage)) == 1


def test_artifact() -> None:
    first, builder = round_trip_for(AwsSagemakerArtifact)
    assert len(builder.resources_of(AwsSagemakerArtifact)) == 1


def test_user_profile() -> None:
    first, builder = round_trip_for(AwsSagemakerUserProfile)
    assert len(builder.resources_of(AwsSagemakerUserProfile)) == 1


def test_pipeline() -> None:
    first, builder = round_trip_for(AwsSagemakerPipeline)
    assert len(builder.resources_of(AwsSagemakerPipeline)) == 1


def test_workteam() -> None:
    first, builder = round_trip_for(AwsSagemakerWorkteam)
    assert len(builder.resources_of(AwsSagemakerWorkteam)) == 1


def test_auto_ml_job() -> None:
    first, builder = round_trip_for(AwsSagemakerAutoMLJob)
    assert len(builder.resources_of(AwsSagemakerAutoMLJob)) == 1


def test_compilation_job() -> None:
    first, builder = round_trip_for(AwsSagemakerCompilationJob)
    assert len(builder.resources_of(AwsSagemakerCompilationJob)) == 1


def test_edge_packaging_job() -> None:
    first, builder = round_trip_for(AwsSagemakerEdgePackagingJob)
    assert len(builder.resources_of(AwsSagemakerEdgePackagingJob)) == 1


def test_hyper_parameter_tuning_job() -> None:
    first, builder = round_trip_for(AwsSagemakerHyperParameterTuningJob)
    assert len(builder.resources_of(AwsSagemakerHyperParameterTuningJob)) == 1


def test_inference_recommendations_job() -> None:
    first, builder = round_trip_for(AwsSagemakerInferenceRecommendationsJob)
    assert len(builder.resources_of(AwsSagemakerInferenceRecommendationsJob)) == 1


def test_labeling_job() -> None:
    first, builder = round_trip_for(AwsSagemakerLabelingJob)
    assert len(builder.resources_of(AwsSagemakerLabelingJob)) == 1
    assert len(first.tags) == 1


def test_processing_job() -> None:
    first, builder = round_trip_for(AwsSagemakerProcessingJob)
    assert len(builder.resources_of(AwsSagemakerProcessingJob)) == 1


def test_training_job() -> None:
    first, builder = round_trip_for(AwsSagemakerTrainingJob)
    assert len(builder.resources_of(AwsSagemakerTrainingJob)) == 1


def test_transform_job() -> None:
    first, builder = round_trip_for(AwsSagemakerTransformJob)
    assert len(builder.resources_of(AwsSagemakerTransformJob)) == 1



from resotolib.proc import num_default_threads
from resotolib.config import Config
from resoto_plugin_aws import AWSCollectorPlugin
from resoto_plugin_aws.configuration import AwsConfig


def test_default_config() -> None:
    config = Config("dummy", "dummy")
    AWSCollectorPlugin.add_config(config)
    Config.init_default_config()
    assert Config.aws.access_key_id is None
    assert Config.aws.secret_access_key is None
    assert Config.aws.role is None
    assert Config.aws.role_override is False
    assert Config.aws.account is None
    assert Config.aws.region is None
    assert Config.aws.scrape_org is False
    assert Config.aws.fork_process is True
    assert Config.aws.scrape_exclude_account == []
    assert Config.aws.assume_current is False
    assert Config.aws.do_not_scrape_current is False
    assert Config.aws.account_pool_size == num_default_threads()
    assert len(Config.aws.collect) == 0
    assert len(Config.aws.no_collect) == 0


def test_session() -> None:
    config = AwsConfig("test", "test", "test")
    # direct session
    assert config.sessions()._session("1234", aws_role=None) == config.sessions()._session("1234", aws_role=None)
    # no test for sts session, since this requires sts setup


def test_shared_tasks_per_key() -> None:
    config = AwsConfig(
        "test", "test", "test", resource_pool_tasks_per_service_default=20, resource_pool_tasks_per_service={"test": 3}
    )
    config.resource_pool_tasks_per_service["sagemaker"] = 6  # predefined
    tpk = config.shared_tasks_per_key(["eu-central-1"])
    assert tpk("eu-central-1:foo") == 20  # default
    assert tpk("eu-central-1:sagemaker") == 6  # predefined
    assert tpk("eu-central-1:test") == 3  # defined in config



from resoto_plugin_aws.resource.efs import AwsEfsFileSystem, AwsEfsMountTarget, AwsEfsAccessPoint
from test.resources import round_trip_for


def test_efs_filesystem() -> None:
    first, builder = round_trip_for(AwsEfsFileSystem, "share_iops")
    assert len(builder.resources_of(AwsEfsMountTarget)) == 2


def test_efs_access_points() -> None:
    round_trip_for(AwsEfsAccessPoint)
